# 14 "src/why3doc/doc_lexer.mll"
 

  open Format
  open Lexing
  open Why3

  let output_comments = ref true

  let newline lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <-
      { pos with pos_lnum = pos.pos_lnum + 1; pos_bol = pos.pos_cnum }

  let make_table l =
    let ht = Hashtbl.create 97 in
    List.iter (fun s -> Hashtbl.add ht s ()) l;
    Hashtbl.mem ht

  let is_keyword1 = make_table [ "as"; "axiom"; "clone"; "coinductive";
    "constant"; "else"; "end"; "epsilon"; "exists"; "export"; "false";
    "forall"; "function"; "goal"; "if"; "import"; "in"; "inductive";
    "lemma"; "let"; "match"; "meta"; "namespace"; "not"; "predicate";
    "prop"; "then"; "theory"; "true"; "type"; "use"; "with";
    (* programs *) "abstract"; "any";
    "begin"; "do"; "done"; "downto"; "exception";
    "for"; "fun"; "ghost"; "loop"; "model"; "module";
    "mutable"; "private"; "raise"; "rec";
    "to"; "try"; "val"; "while"; ]

  let is_keyword2 = make_table [ "absurd"; "assert"; "assume";
    "ensures"; "check"; "invariant"; "raises"; "reads"; "requires";
    "returns"; "variant"; "writes"; ]

  let get_loc lb =
    let p = Lexing.lexeme_start_p lb in
    p.pos_fname, p.pos_lnum, p.pos_cnum - p.pos_bol

  let html_char fmt c =
    pp_print_string fmt (match c with
      | '<' -> "&lt;"
      | '>' -> "&gt;"
      | '&' -> "&amp;"
      | _ -> assert false)

  let html_escape s =
    let len = ref 0 in
    String.iter (function '<' | '>' -> len := !len + 4
      | '&' -> len := !len + 5 | _ -> incr len) s;
    let len' = !len in
    let len = String.length s in
    if len = len' then s else begin
      let t = String.create len' in
      let j = ref 0 in
      let app u =
        let l = String.length u in
        String.blit u 0 t !j l;
        j := !j + l in
      for i = 0 to len - 1 do
        match s.[i] with
        | '<' -> app "&lt;"
        | '>' -> app "&gt;"
        | '&' -> app "&amp;"
        | c -> t.[!j] <- c; incr j
      done;
      t
    end

  let current_file = ref ""

  let print_ident fmt lexbuf s =
    if is_keyword1 s then
      fprintf fmt "<span class=\"keyword1\">%s</span>" s
    else if is_keyword2 s then
      fprintf fmt "<span class=\"keyword2\">%s</span>" s
    else begin
      let (* f,l,c as *) loc = get_loc lexbuf in
      (* Format.eprintf "  IDENT %s/%d/%d@." f l c; *)
      (* is this a def point? *)
      let s = html_escape s in
      try
        let t = Doc_def.is_def loc in
        fprintf fmt "<a name=\"%s\">%s</a>" t s
      with Not_found ->
      (* is this a use point? *)
      try
        let id = Glob.locate loc in
        let fn, url, t = Doc_def.locate id in
        let url = if fn = !current_file then "" else url in
        fprintf fmt "<a href=\"%s#%s\">%s</a>" url t s
      with Not_found ->
        (* otherwise, just print it *)
        pp_print_string fmt s
    end


# 98 "src/why3doc/doc_lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\247\255\007\000\248\255\249\255\032\000\094\000\133\000\
    \179\000\225\000\008\001\070\001\251\255\012\000\006\000\008\000\
    \009\000\255\255\254\255\011\000\102\000\012\000\250\255\010\000\
    \033\000\011\000\003\000\018\000\183\001\249\255\025\000\250\255\
    \251\255\019\002\058\002\104\002\143\002\189\002\222\002\253\255\
    \254\255\068\000\078\000\073\000\090\000\252\255\045\000\104\000\
    \050\001\249\255\110\000\250\255\251\255\252\255\253\255\104\000\
    \104\000\255\255\254\255\108\000\071\001\252\255\008\000\253\255\
    \254\255\255\255\096\002\244\255\245\255\247\255\248\255\132\000\
    \001\000\254\255\107\000\117\000\109\000\121\000\255\255\253\255\
    \250\255\119\000\198\000\251\255\252\255\253\255\254\255\255\255\
    \191\000\252\255\254\255\155\000\118\000\120\000\158\000\153\000\
    \040\000\255\255\204\000\252\255\253\255\130\000\132\000\255\255\
    \254\255";
  Lexing.lex_backtrk = 
   "\255\255\255\255\008\000\255\255\255\255\008\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\255\255\008\000\008\000\003\000\
    \002\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \005\000\255\255\255\255\255\255\255\255\255\255\006\000\255\255\
    \255\255\003\000\003\000\003\000\003\000\003\000\003\000\255\255\
    \255\255\000\000\255\255\003\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\006\000\255\255\255\255\255\255\255\255\006\000\
    \006\000\255\255\255\255\255\255\255\255\255\255\003\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\006\000\
    \003\000\255\255\011\000\009\000\255\255\255\255\255\255\255\255\
    \255\255\004\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\001\000\003\000\002\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\003\000\003\000\255\255\
    \255\255";
  Lexing.lex_default = 
   "\001\000\000\000\255\255\000\000\000\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\000\000\000\000\255\255\255\255\255\255\000\000\255\255\
    \255\255\255\255\255\255\255\255\029\000\000\000\255\255\000\000\
    \000\000\255\255\255\255\255\255\255\255\255\255\255\255\000\000\
    \000\000\255\255\255\255\255\255\255\255\000\000\255\255\255\255\
    \049\000\000\000\255\255\000\000\000\000\000\000\000\000\255\255\
    \255\255\000\000\000\000\255\255\061\000\000\000\061\000\000\000\
    \000\000\000\000\067\000\000\000\000\000\000\000\000\000\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\000\000\000\000\
    \000\000\255\255\083\000\000\000\000\000\000\000\000\000\000\000\
    \089\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \096\000\000\000\099\000\000\000\000\000\255\255\255\255\000\000\
    \000\000";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\004\000\079\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \013\000\010\000\003\000\006\000\006\000\007\000\006\000\002\000\
    \014\000\027\000\007\000\008\000\020\000\008\000\006\000\007\000\
    \015\000\017\000\016\000\018\000\019\000\021\000\016\000\026\000\
    \022\000\001\000\006\000\047\000\009\000\009\000\009\000\010\000\
    \006\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\005\000\023\000\025\000\006\000\011\000\
    \022\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\046\000\006\000\024\000\009\000\006\000\
    \042\000\006\000\006\000\007\000\006\000\044\000\020\000\045\000\
    \007\000\008\000\045\000\008\000\006\000\007\000\019\000\029\000\
    \059\000\058\000\057\000\049\000\078\000\077\000\078\000\081\000\
    \006\000\077\000\009\000\009\000\009\000\006\000\006\000\076\000\
    \093\000\043\000\094\000\076\000\091\000\097\000\007\000\094\000\
    \007\000\007\000\007\000\007\000\104\000\103\000\000\000\007\000\
    \008\000\000\000\008\000\007\000\007\000\081\000\081\000\081\000\
    \081\000\081\000\081\000\091\000\006\000\022\000\094\000\007\000\
    \000\000\009\000\009\000\009\000\007\000\007\000\000\000\000\000\
    \091\000\090\000\096\000\096\000\096\000\096\000\096\000\096\000\
    \086\000\000\000\000\000\000\000\008\000\000\000\008\000\008\000\
    \008\000\008\000\006\000\000\000\009\000\008\000\008\000\091\000\
    \008\000\008\000\008\000\007\000\022\000\000\000\000\000\092\000\
    \000\000\000\000\000\000\000\000\080\000\008\000\000\000\009\000\
    \009\000\009\000\008\000\008\000\101\000\000\000\102\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \012\000\007\000\009\000\009\000\009\000\009\000\009\000\009\000\
    \255\255\000\000\000\000\009\000\009\000\000\000\009\000\009\000\
    \009\000\008\000\022\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\095\000\000\000\009\000\000\000\009\000\009\000\009\000\
    \009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \255\255\010\000\000\000\010\000\010\000\007\000\010\000\008\000\
    \000\000\009\000\007\000\008\000\000\000\008\000\010\000\007\000\
    \000\000\000\000\000\000\000\000\053\000\000\000\000\000\009\000\
    \022\000\085\000\010\000\084\000\009\000\009\000\009\000\010\000\
    \010\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\065\000\000\000\000\000\052\000\000\000\000\000\000\000\
    \051\000\050\000\056\000\000\000\055\000\009\000\000\000\009\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\010\000\022\000\
    \000\000\064\000\000\000\000\000\000\000\063\000\051\000\000\000\
    \051\000\000\000\000\000\000\000\000\000\000\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \000\000\000\000\000\000\063\000\010\000\063\000\009\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\000\000\000\000\062\000\000\000\011\000\000\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\032\000\000\000\000\000\000\000\000\000\087\000\000\000\
    \000\000\000\000\000\000\000\000\100\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \037\000\031\000\033\000\033\000\034\000\033\000\030\000\000\000\
    \000\000\034\000\035\000\000\000\035\000\033\000\034\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\033\000\000\000\036\000\036\000\036\000\037\000\033\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\041\000\000\000\040\000\033\000\038\000\000\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\054\000\033\000\033\000\036\000\033\000\033\000\
    \034\000\033\000\000\000\000\000\000\000\034\000\035\000\000\000\
    \035\000\033\000\034\000\000\000\000\000\000\000\000\000\255\255\
    \000\000\000\000\000\000\000\000\000\000\033\000\000\000\036\000\
    \036\000\036\000\033\000\033\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\034\000\000\000\034\000\034\000\034\000\
    \034\000\000\000\000\000\000\000\034\000\035\000\000\000\035\000\
    \034\000\034\000\072\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\033\000\045\000\000\000\034\000\000\000\036\000\036\000\
    \036\000\034\000\034\000\000\000\000\000\000\000\000\000\000\000\
    \075\000\000\000\000\000\000\000\000\000\000\000\068\000\000\000\
    \000\000\035\000\074\000\035\000\035\000\035\000\035\000\033\000\
    \000\000\036\000\035\000\035\000\000\000\035\000\035\000\035\000\
    \034\000\045\000\000\000\000\000\068\000\000\000\068\000\000\000\
    \000\000\000\000\035\000\000\000\036\000\036\000\036\000\035\000\
    \035\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \036\000\000\000\036\000\036\000\036\000\036\000\034\000\039\000\
    \036\000\036\000\036\000\069\000\036\000\036\000\036\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\035\000\045\000\
    \000\000\036\000\000\000\036\000\036\000\036\000\036\000\036\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\071\000\000\000\070\000\037\000\000\000\
    \037\000\037\000\034\000\037\000\035\000\000\000\036\000\034\000\
    \035\000\000\000\035\000\037\000\034\000\036\000\045\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\037\000\
    \000\000\036\000\036\000\036\000\037\000\037\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\036\000\000\000\036\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \000\000\000\000\000\000\037\000\045\000\000\000\000\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\037\000\000\000\036\000\000\000\038\000\000\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \073\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\000\000\072\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\002\000\000\000\000\000\013\000\000\000\000\000\000\000\
    \014\000\015\000\015\000\016\000\013\000\019\000\021\000\023\000\
    \025\000\027\000\000\000\030\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\005\000\024\000\000\000\000\000\
    \026\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\042\000\000\000\005\000\000\000\006\000\
    \041\000\006\000\006\000\006\000\006\000\043\000\020\000\044\000\
    \006\000\006\000\046\000\006\000\006\000\006\000\020\000\047\000\
    \050\000\055\000\056\000\059\000\074\000\075\000\076\000\081\000\
    \006\000\077\000\006\000\006\000\006\000\006\000\006\000\075\000\
    \092\000\041\000\093\000\077\000\091\000\096\000\007\000\094\000\
    \007\000\007\000\007\000\007\000\101\000\102\000\255\255\007\000\
    \007\000\255\255\007\000\007\000\007\000\071\000\071\000\071\000\
    \071\000\071\000\071\000\091\000\006\000\006\000\094\000\007\000\
    \255\255\007\000\007\000\007\000\007\000\007\000\255\255\255\255\
    \088\000\088\000\095\000\095\000\095\000\095\000\095\000\095\000\
    \082\000\255\255\255\255\255\255\008\000\255\255\008\000\008\000\
    \008\000\008\000\006\000\255\255\006\000\008\000\008\000\088\000\
    \008\000\008\000\008\000\007\000\007\000\255\255\255\255\088\000\
    \255\255\255\255\255\255\255\255\071\000\008\000\255\255\008\000\
    \008\000\008\000\008\000\008\000\098\000\255\255\098\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\007\000\009\000\007\000\009\000\009\000\009\000\009\000\
    \062\000\255\255\255\255\009\000\009\000\255\255\009\000\009\000\
    \009\000\008\000\008\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\094\000\255\255\009\000\255\255\009\000\009\000\009\000\
    \009\000\009\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \096\000\010\000\255\255\010\000\010\000\010\000\010\000\008\000\
    \255\255\008\000\010\000\010\000\255\255\010\000\010\000\010\000\
    \255\255\255\255\255\255\255\255\048\000\255\255\255\255\009\000\
    \009\000\082\000\010\000\082\000\010\000\010\000\010\000\010\000\
    \010\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\060\000\255\255\255\255\048\000\255\255\255\255\255\255\
    \048\000\048\000\048\000\255\255\048\000\009\000\255\255\009\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\010\000\010\000\
    \255\255\060\000\255\255\255\255\255\255\060\000\048\000\255\255\
    \048\000\255\255\255\255\255\255\255\255\255\255\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \255\255\255\255\255\255\060\000\010\000\060\000\010\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\255\255\255\255\060\000\255\255\011\000\255\255\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\028\000\255\255\255\255\255\255\255\255\082\000\255\255\
    \255\255\255\255\255\255\255\255\098\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\255\255\
    \255\255\028\000\028\000\255\255\028\000\028\000\028\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\028\000\255\255\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\028\000\255\255\028\000\028\000\028\000\255\255\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\048\000\028\000\033\000\028\000\033\000\033\000\
    \033\000\033\000\255\255\255\255\255\255\033\000\033\000\255\255\
    \033\000\033\000\033\000\255\255\255\255\255\255\255\255\060\000\
    \255\255\255\255\255\255\255\255\255\255\033\000\255\255\033\000\
    \033\000\033\000\033\000\033\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\034\000\255\255\034\000\034\000\034\000\
    \034\000\255\255\255\255\255\255\034\000\034\000\255\255\034\000\
    \034\000\034\000\066\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\033\000\033\000\255\255\034\000\255\255\034\000\034\000\
    \034\000\034\000\034\000\255\255\255\255\255\255\255\255\255\255\
    \066\000\255\255\255\255\255\255\255\255\255\255\066\000\255\255\
    \255\255\035\000\066\000\035\000\035\000\035\000\035\000\033\000\
    \255\255\033\000\035\000\035\000\255\255\035\000\035\000\035\000\
    \034\000\034\000\255\255\255\255\066\000\255\255\066\000\255\255\
    \255\255\255\255\035\000\255\255\035\000\035\000\035\000\035\000\
    \035\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \036\000\255\255\036\000\036\000\036\000\036\000\034\000\028\000\
    \034\000\036\000\036\000\066\000\036\000\036\000\036\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\035\000\035\000\
    \255\255\036\000\255\255\036\000\036\000\036\000\036\000\036\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\066\000\255\255\066\000\037\000\255\255\
    \037\000\037\000\037\000\037\000\035\000\255\255\035\000\037\000\
    \037\000\255\255\037\000\037\000\037\000\036\000\036\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\037\000\
    \255\255\037\000\037\000\037\000\037\000\037\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\036\000\255\255\036\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \255\255\255\255\255\255\037\000\037\000\255\255\255\255\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\037\000\255\255\037\000\255\255\038\000\255\255\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
    \038\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \066\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_base_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\004\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_backtrk_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_default_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_trans_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\001\000\001\000\001\000\001\000\001\000\001\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_check_code = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\095\000\095\000\095\000\095\000\095\000\095\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255";
  Lexing.lex_code = 
   "\255\001\255\255\000\001\255";
}

let rec scan fmt lexbuf =
    __ocaml_lex_scan_rec fmt lexbuf 0
and __ocaml_lex_scan_rec fmt lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
let
# 136 "src/why3doc/doc_lexer.mll"
             s
# 520 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos (lexbuf.Lexing.lex_start_pos + 3) in
# 137 "src/why3doc/doc_lexer.mll"
          ( pp_print_string fmt s;
            scan fmt lexbuf )
# 525 "src/why3doc/doc_lexer.ml"

  | 1 ->
# 140 "src/why3doc/doc_lexer.mll"
          ( comment fmt false lexbuf;
            scan fmt lexbuf )
# 531 "src/why3doc/doc_lexer.ml"

  | 2 ->
# 143 "src/why3doc/doc_lexer.mll"
          ( fprintf fmt "</pre>@\n";
            doc fmt false [] lexbuf;
            fprintf fmt "<pre>";
            scan fmt lexbuf )
# 539 "src/why3doc/doc_lexer.ml"

  | 3 ->
# 148 "src/why3doc/doc_lexer.mll"
          ( fprintf fmt "<span class=\"comment\">(*";
            comment fmt true lexbuf;
            fprintf fmt "</span>";
            scan fmt lexbuf )
# 547 "src/why3doc/doc_lexer.ml"

  | 4 ->
# 152 "src/why3doc/doc_lexer.mll"
          ( () )
# 552 "src/why3doc/doc_lexer.ml"

  | 5 ->
let
# 153 "src/why3doc/doc_lexer.mll"
             s
# 558 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 154 "src/why3doc/doc_lexer.mll"
          ( print_ident fmt lexbuf s;
            scan fmt lexbuf )
# 563 "src/why3doc/doc_lexer.ml"

  | 6 ->
# 156 "src/why3doc/doc_lexer.mll"
           ( newline lexbuf;
             fprintf fmt "\n";
             scan fmt lexbuf )
# 570 "src/why3doc/doc_lexer.ml"

  | 7 ->
# 159 "src/why3doc/doc_lexer.mll"
           ( fprintf fmt "&quot;";
             string fmt true lexbuf;
             scan fmt lexbuf )
# 577 "src/why3doc/doc_lexer.ml"

  | 8 ->
let
# 163 "src/why3doc/doc_lexer.mll"
         s
# 583 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 163 "src/why3doc/doc_lexer.mll"
           ( pp_print_string fmt s;
             scan fmt lexbuf )
# 588 "src/why3doc/doc_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_scan_rec fmt lexbuf __ocaml_lex_state

and scan_embedded fmt depth lexbuf =
    __ocaml_lex_scan_embedded_rec fmt depth lexbuf 28
and __ocaml_lex_scan_embedded_rec fmt depth lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 167 "src/why3doc/doc_lexer.mll"
          ( pp_print_char fmt '[';
            scan_embedded fmt (depth + 1) lexbuf )
# 600 "src/why3doc/doc_lexer.ml"

  | 1 ->
# 169 "src/why3doc/doc_lexer.mll"
          ( if depth > 0 then begin
              pp_print_char fmt ']';
              scan_embedded fmt (depth - 1) lexbuf
            end )
# 608 "src/why3doc/doc_lexer.ml"

  | 2 ->
# 173 "src/why3doc/doc_lexer.mll"
          ( () )
# 613 "src/why3doc/doc_lexer.ml"

  | 3 ->
let
# 174 "src/why3doc/doc_lexer.mll"
             s
# 619 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 175 "src/why3doc/doc_lexer.mll"
          ( print_ident fmt lexbuf s;
            scan_embedded fmt depth lexbuf )
# 624 "src/why3doc/doc_lexer.ml"

  | 4 ->
# 177 "src/why3doc/doc_lexer.mll"
           ( newline lexbuf;
             fprintf fmt "\n";
             scan_embedded fmt depth lexbuf )
# 631 "src/why3doc/doc_lexer.ml"

  | 5 ->
# 180 "src/why3doc/doc_lexer.mll"
           ( fprintf fmt "&quot;";
             string fmt true lexbuf;
             scan_embedded fmt depth lexbuf )
# 638 "src/why3doc/doc_lexer.ml"

  | 6 ->
let
# 184 "src/why3doc/doc_lexer.mll"
         s
# 644 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 184 "src/why3doc/doc_lexer.mll"
           ( pp_print_string fmt s;
             scan_embedded fmt depth lexbuf )
# 649 "src/why3doc/doc_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_scan_embedded_rec fmt depth lexbuf __ocaml_lex_state

and comment fmt do_output lexbuf =
    __ocaml_lex_comment_rec fmt do_output lexbuf 48
and __ocaml_lex_comment_rec fmt do_output lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 188 "src/why3doc/doc_lexer.mll"
           ( if do_output then fprintf fmt "(*";
             comment fmt do_output lexbuf;
             comment fmt do_output lexbuf )
# 662 "src/why3doc/doc_lexer.ml"

  | 1 ->
# 191 "src/why3doc/doc_lexer.mll"
           ( if do_output then fprintf fmt "*)" )
# 667 "src/why3doc/doc_lexer.ml"

  | 2 ->
# 192 "src/why3doc/doc_lexer.mll"
           ( () )
# 672 "src/why3doc/doc_lexer.ml"

  | 3 ->
# 193 "src/why3doc/doc_lexer.mll"
           ( newline lexbuf;
             if do_output then fprintf fmt "\n";
             comment fmt do_output lexbuf )
# 679 "src/why3doc/doc_lexer.ml"

  | 4 ->
# 196 "src/why3doc/doc_lexer.mll"
           ( if do_output then fprintf fmt "&quot;";
             string fmt do_output lexbuf;
             comment fmt do_output lexbuf )
# 686 "src/why3doc/doc_lexer.ml"

  | 5 ->
let
# 199 "src/why3doc/doc_lexer.mll"
               c
# 692 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 200 "src/why3doc/doc_lexer.mll"
           ( if do_output then html_char fmt c;
             comment fmt do_output lexbuf )
# 697 "src/why3doc/doc_lexer.ml"

  | 6 ->
let
# 203 "src/why3doc/doc_lexer.mll"
         s
# 703 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 203 "src/why3doc/doc_lexer.mll"
           ( if do_output then pp_print_string fmt s;
             comment fmt do_output lexbuf )
# 708 "src/why3doc/doc_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_comment_rec fmt do_output lexbuf __ocaml_lex_state

and string fmt do_output lexbuf =
    __ocaml_lex_string_rec fmt do_output lexbuf 60
and __ocaml_lex_string_rec fmt do_output lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 207 "src/why3doc/doc_lexer.mll"
           ( newline lexbuf;
             if do_output then fprintf fmt "\n";
             string fmt do_output lexbuf )
# 721 "src/why3doc/doc_lexer.ml"

  | 1 ->
# 210 "src/why3doc/doc_lexer.mll"
           ( if do_output then fprintf fmt "&quot;" )
# 726 "src/why3doc/doc_lexer.ml"

  | 2 ->
let
# 211 "src/why3doc/doc_lexer.mll"
               c
# 732 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 212 "src/why3doc/doc_lexer.mll"
           ( if do_output then html_char fmt c;
             string fmt do_output lexbuf )
# 737 "src/why3doc/doc_lexer.ml"

  | 3 ->
let
# 215 "src/why3doc/doc_lexer.mll"
         s
# 743 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 215 "src/why3doc/doc_lexer.mll"
           ( if do_output then pp_print_string fmt s;
             string fmt do_output lexbuf )
# 748 "src/why3doc/doc_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_string_rec fmt do_output lexbuf __ocaml_lex_state

and doc fmt block headings lexbuf =
    __ocaml_lex_doc_rec fmt block headings lexbuf 66
and __ocaml_lex_doc_rec fmt block headings lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 219 "src/why3doc/doc_lexer.mll"
                ( if block then fprintf fmt "</p>@\n" )
# 759 "src/why3doc/doc_lexer.ml"

  | 1 ->
# 220 "src/why3doc/doc_lexer.mll"
           ( () )
# 764 "src/why3doc/doc_lexer.ml"

  | 2 ->
# 221 "src/why3doc/doc_lexer.mll"
           ( newline lexbuf;
             newline lexbuf;
             if block then pp_print_string fmt "</p>";
             fprintf fmt "@\n";
             doc fmt false headings lexbuf )
# 773 "src/why3doc/doc_lexer.ml"

  | 3 ->
# 226 "src/why3doc/doc_lexer.mll"
           ( newline lexbuf;
             fprintf fmt "\n";
             doc fmt block headings lexbuf )
# 780 "src/why3doc/doc_lexer.ml"

  | 4 ->
let
# 229 "src/why3doc/doc_lexer.mll"
                      c
# 786 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_start_pos + 1) in
# 230 "src/why3doc/doc_lexer.mll"
           ( if block then fprintf fmt "</p>@\n";
             let n = Char.code c - Char.code '0' in
             fprintf fmt "<h%d>" n;
             doc fmt true (n::headings) lexbuf )
# 793 "src/why3doc/doc_lexer.ml"

  | 5 ->
# 234 "src/why3doc/doc_lexer.mll"
           ( if not block then pp_print_string fmt "<p>";
             raw_html fmt 0 lexbuf; doc fmt true headings lexbuf )
# 799 "src/why3doc/doc_lexer.ml"

  | 6 ->
# 236 "src/why3doc/doc_lexer.mll"
           ( if not block then pp_print_string fmt "<p>";
             pp_print_char fmt '{';
             doc fmt true (0::headings) lexbuf )
# 806 "src/why3doc/doc_lexer.ml"

  | 7 ->
# 239 "src/why3doc/doc_lexer.mll"
           ( let brace r =
               if not block then pp_print_string fmt "<p>";
               fprintf fmt "}";
               doc fmt true r lexbuf
             in
             match headings with
              | [] -> brace headings
              | n :: r ->
                if n >= 1 then begin
                  fprintf fmt "</h%d>" n;
                  doc fmt (r <> []) r lexbuf
                end else brace r
           )
# 823 "src/why3doc/doc_lexer.ml"

  | 8 ->
# 252 "src/why3doc/doc_lexer.mll"
           ( if not block then pp_print_string fmt "<p>";
             pp_print_string fmt "<code>";
             scan_embedded fmt 0 lexbuf;
             pp_print_string fmt "</code>";
             doc fmt true headings lexbuf )
# 832 "src/why3doc/doc_lexer.ml"

  | 9 ->
# 257 "src/why3doc/doc_lexer.mll"
           ( if block then pp_print_char fmt ' ';
             doc fmt block headings lexbuf )
# 838 "src/why3doc/doc_lexer.ml"

  | 10 ->
let
# 259 "src/why3doc/doc_lexer.mll"
               c
# 844 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 260 "src/why3doc/doc_lexer.mll"
           ( if not block then pp_print_string fmt "<p>";
             html_char fmt c;
             doc fmt true headings lexbuf )
# 850 "src/why3doc/doc_lexer.ml"

  | 11 ->
let
# 263 "src/why3doc/doc_lexer.mll"
         c
# 856 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 263 "src/why3doc/doc_lexer.mll"
           ( if not block then pp_print_string fmt "<p>";
             pp_print_char fmt c;
             doc fmt true headings lexbuf )
# 862 "src/why3doc/doc_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_doc_rec fmt block headings lexbuf __ocaml_lex_state

and raw_html fmt depth lexbuf =
    __ocaml_lex_raw_html_rec fmt depth lexbuf 82
and __ocaml_lex_raw_html_rec fmt depth lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 269 "src/why3doc/doc_lexer.mll"
           ( () )
# 873 "src/why3doc/doc_lexer.ml"

  | 1 ->
# 270 "src/why3doc/doc_lexer.mll"
           ( newline lexbuf;
             fprintf fmt "\n";
             raw_html fmt depth lexbuf )
# 880 "src/why3doc/doc_lexer.ml"

  | 2 ->
# 273 "src/why3doc/doc_lexer.mll"
           ( fprintf fmt "{"; raw_html fmt (succ depth) lexbuf )
# 885 "src/why3doc/doc_lexer.ml"

  | 3 ->
# 274 "src/why3doc/doc_lexer.mll"
           ( if depth = 0 then () else
               begin
                 fprintf fmt "}";
                 raw_html fmt (pred depth) lexbuf
               end )
# 894 "src/why3doc/doc_lexer.ml"

  | 4 ->
let
# 279 "src/why3doc/doc_lexer.mll"
         c
# 900 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 279 "src/why3doc/doc_lexer.mll"
           ( pp_print_char fmt c; raw_html fmt depth lexbuf )
# 904 "src/why3doc/doc_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_raw_html_rec fmt depth lexbuf __ocaml_lex_state

and extract_header lexbuf =
  lexbuf.Lexing.lex_mem <- Array.create 2 (-1) ;   __ocaml_lex_extract_header_rec lexbuf 88
and __ocaml_lex_extract_header_rec lexbuf __ocaml_lex_state =
  match Lexing.new_engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
let
# 283 "src/why3doc/doc_lexer.mll"
                                            header
# 916 "src/why3doc/doc_lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) (lexbuf.Lexing.lex_curr_pos + -1) in
# 284 "src/why3doc/doc_lexer.mll"
      ( header )
# 920 "src/why3doc/doc_lexer.ml"

  | 1 ->
# 286 "src/why3doc/doc_lexer.mll"
      ( extract_header lexbuf )
# 925 "src/why3doc/doc_lexer.ml"

  | 2 ->
# 288 "src/why3doc/doc_lexer.mll"
      ( skip_comment lexbuf; extract_header lexbuf )
# 930 "src/why3doc/doc_lexer.ml"

  | 3 ->
# 290 "src/why3doc/doc_lexer.mll"
      ( "" )
# 935 "src/why3doc/doc_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_extract_header_rec lexbuf __ocaml_lex_state

and skip_comment lexbuf =
    __ocaml_lex_skip_comment_rec lexbuf 98
and __ocaml_lex_skip_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 293 "src/why3doc/doc_lexer.mll"
         ( () )
# 946 "src/why3doc/doc_lexer.ml"

  | 1 ->
# 294 "src/why3doc/doc_lexer.mll"
         ( skip_comment lexbuf; skip_comment lexbuf )
# 951 "src/why3doc/doc_lexer.ml"

  | 2 ->
# 295 "src/why3doc/doc_lexer.mll"
         ( () )
# 956 "src/why3doc/doc_lexer.ml"

  | 3 ->
# 296 "src/why3doc/doc_lexer.mll"
         ( skip_comment lexbuf )
# 961 "src/why3doc/doc_lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_skip_comment_rec lexbuf __ocaml_lex_state

;;

# 298 "src/why3doc/doc_lexer.mll"
 

  let do_file fmt fname =
    current_file := fname;
    (* input *)
    let cin = open_in fname in
    let lb = Lexing.from_channel cin in
    lb.Lexing.lex_curr_p <-
      { lb.Lexing.lex_curr_p with Lexing.pos_fname = fname };
    (* output *)
    fprintf fmt "<div class=\"why3doc\">@\n";
    fprintf fmt "<pre>@\n";
    scan fmt lb;
    fprintf fmt "</pre>@\n";
    fprintf fmt "</div>@\n";
    close_in cin

  let extract_header fname =
    let cin = open_in fname in
    let lb = Lexing.from_channel cin in
    let h = extract_header lb in
    close_in cin;
    h


# 993 "src/why3doc/doc_lexer.ml"
